var isDirty = false

function ReadSetting( settingName, defaultValue ) {
  var val = System.Gadget.Settings.read( settingName )
  if ( val != "" ) return val
  return defaultValue
}


function readDrivesSetting() {
  var drives = ReadSetting( "drives", "C" ) 
  if ( !drives ) return;

  document.numDrives = drives.length

  drivearea.innerHTML = ""
  for ( var i=0; i<drives.length; i++ ) {
    drivearea.innerHTML += "<div id=statusArea"+i+" class=statusArea><img id=statusBar"+i+" class=statusBar /></div><div id=label"+i+" onclick='openDrive(this)' class=freeSpaceText>--</div>";
    var textSel = document.getElementById( "label" + i )
    if ( !textSel) continue
    textSel.innerText = drives.charAt(i)
  }
}

function readSettings() {
  readDrivesSetting()
  changeGadgetSize()
  displayAllUsage()
}

function changeGadgetSize() {
  var minSize = 67
  var oneLine = 13

  var newheight = minSize
  var extraDrives = document.numDrives - 4
  if ( extraDrives > 0 ) {
    newheight += oneLine * extraDrives
  }

  document.body.style.height = newheight
  background.style.height = newheight
}

function startup() {
  System.Gadget.settingsUI = "settings.html"
  System.Gadget.onSettingsClosed = readSettings
  System.Gadget.visibilityChanged = checkVisibility

  readSettings()
  window.setInterval(updateGadget, 5000)
}

function updateGadget() {
  if ( ! System.Gadget.visible ) {
    isDirty = true
  } else {
    displayAllUsage();
    isDirty = false
  }
}

function checkVisibility() {
  // See http://blogs.msdn.com/sidebar/archive/2006/08/18/706495.aspx
  if ( System.Gadget.visible && isDirty ) {
    updateGadget()
  }
}

function displayAllUsage() {
  for ( i=0; i<document.numDrives; i++ ) {
    statusEl = document.getElementById( "statusBar" + i )
    textEl = document.getElementById( "label" + i )
    driveLetter = textEl.innerText.substring(0,1)
    if ( statusEl && textEl && driveLetter != "-" ) {
      displayUsage( driveLetter, statusEl, textEl )
    }
  }
}

function displayUsage( driveLetter, statusBarEl, freeSpaceTextEl ) {
  try {
    var curDrive = System.Shell.drive( driveLetter )
    var freeSpace = Math.round(curDrive.freeSpace)
    var friendlyFreeSpace = convertBytes( freeSpace )
    var percentUsed = 1 - curDrive.freeSpace / curDrive.totalSize
  } catch(err) {
    freeSpaceTextEl.innerText = driveLetter + ": unavail"
    return
  }

  freeSpaceTextEl.innerText = driveLetter + ": " + friendlyFreeSpace
			
  critical = ReadSetting( "critical", 95 ) / 100
  warning = ReadSetting( "warning", 90 ) / 100

  if (percentUsed < warning) {
    statusBarEl.src = "images/blue.png"
  } else if ( percentUsed < critical ) {
    statusBarEl.src = "images/roux.png"
  } else {
    statusBarEl.src = "images/red.png"
  } 
			
  statusBarEl.width = 50 * percentUsed

  var friendlyPercentUsed = Math.round( 100*percentUsed )
  var friendlyPercentFree = 100 - Math.round( 100*percentUsed )
  var friendlyTotal = convertBytes(curDrive.totalSize);
  var friendlyUsedSpace = convertBytes( curDrive.totalSize - curDrive.freeSpace )
  statusBarEl.title = "Total size: " + friendlyTotal + "\n" +
    "Free: " + friendlyFreeSpace + " (" + friendlyPercentFree + "%)\n" + 
    "Used: " + friendlyUsedSpace + " (" + friendlyPercentUsed + "%)"
}

function convertBytes( bytes ) {
  var i = 0;
  var units = Array( " MB", " GB", " TB" );

  while ( bytes >= 1024 && ( bytes / 1024 ) >= 1 ) {
    bytes /= 1024
    i++
  }

  return ( Math.round(bytes) ) + units[i]
}

function openDrive( textEl ) {
  var driveLetter = textEl.innerText.substring(0,1)
  System.Shell.execute( driveLetter + ":" )
}

function init_settings() {
  document.getElementById("drives").value = ReadSetting( "drives", "C" )
  document.getElementById("critical").value = ReadSetting( "critical", 95 )
  document.getElementById("warning").value = ReadSetting( "warning", 90 )
}

function CheckAndSetDrives() {
  var drives = document.getElementById("drives").value
  drives = drives.replace( /[^A-Za-z]+/g, "" )
  if ( drives.length == 0 ) drives = "C"
  System.Gadget.Settings.write( "drives", drives )
}

function CheckAndSetThreshold( thresh ) {
  var sel = document.getElementById( thresh )
  var val = sel.value
  if ( val >= 0 && val <= 100 ) {
    System.Gadget.Settings.write( thresh, val )
  }
}

function SettingsClosing(event) {
  if (event.closeAction == event.Action.commit) {
    CheckAndSetDrives()
    CheckAndSetThreshold( "critical" )
    CheckAndSetThreshold( "warning" )
  }
		
  event.cancel = false;
}
